"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useAsyncOnDemand = useAsyncOnDemand;
exports["default"] = void 0;

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _react = require("react");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { (0, _defineProperty2["default"])(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

var noop = function noop() {};

var Action = {
  EXECUTE: 'execute',
  RESOLVE: 'resolve',
  REJECT: 'reject'
};

function taskReducer(state, _ref) {
  var type = _ref.type,
      payload = _ref.payload;

  switch (type) {
    case Action.EXECUTE:
      return _objectSpread({}, state, {
        isPending: true
      });

    case Action.RESOLVE:
      return {
        result: payload,
        isPending: false
      };

    case Action.REJECT:
      return {
        error: payload,
        isPending: false
      };

    /* istanbul ignore next */

    default:
      return state;
  }
}

function initTaskState() {
  return {
    isPending: false
  };
}

function createAbortError() {
  var error = new Error('The operation was aborted');
  error.name = 'AbortError';
  return error;
}

function executeTask(createTask, inputs) {
  var abortController = typeof AbortController !== 'undefined' ? new AbortController() : null;
  var abortSignal = abortController ? abortController.signal : undefined;
  var abortByController = abortController ? function () {
    return abortController.abort();
  } : noop;
  var injection = {
    abortSignal: abortSignal
  };
  var isCancelled = false;

  var cancel = function cancel() {
    isCancelled = true;
    abortByController();
  };

  var proxyPromise;

  try {
    var task = createTask(inputs, injection);
    var taskPromise = task instanceof Promise ? task : Promise.resolve(task);
    proxyPromise = new Promise(function (resolve, reject) {
      taskPromise.then(function (result) {
        return isCancelled ? reject(createAbortError()) : resolve(result);
      }, function (error) {
        return isCancelled ? reject(createAbortError()) : reject(error);
      });
    });
  } catch (error) {
    proxyPromise = Promise.reject(error);
  }

  return {
    cancel: cancel,
    promise: proxyPromise
  };
}

function executeTaskOnEffect(dispatch, createTask, inputs, _ref2) {
  var _ref2$onError = _ref2.onError,
      onError = _ref2$onError === void 0 ? noop : _ref2$onError,
      _ref2$onSuccess = _ref2.onSuccess,
      onSuccess = _ref2$onSuccess === void 0 ? noop : _ref2$onSuccess;
  var task = executeTask(createTask, inputs);
  dispatch({
    type: Action.EXECUTE
  });
  task.promise.then(function (result) {
    dispatch({
      type: Action.RESOLVE,
      payload: result
    });
    onSuccess(result, inputs);
  }, function (error) {
    dispatch({
      type: Action.REJECT,
      payload: error
    });
    onError(error, inputs);
  });
  return task;
}

function useAsyncInternal(createTask, inputs, _ref3) {
  var onError = _ref3.onError,
      onSuccess = _ref3.onSuccess,
      isOnDemand = _ref3.isOnDemand;
  var staticConfig = (0, _react.useRef)({
    isOnDemand: isOnDemand
  }).current;
  var selfRef = (0, _react.useRef)({
    task: {
      cancel: noop
    }
  });
  var self = selfRef.current;
  (0, _react.useEffect)(function () {
    Object.assign(self, {
      onError: onError,
      onSuccess: onSuccess,
      createTask: createTask
    });
  });

  var _useState = (0, _react.useState)(0),
      _useState2 = (0, _slicedToArray2["default"])(_useState, 2),
      count = _useState2[0],
      setCount = _useState2[1];

  var execute = function execute() {
    setCount(function (prevCount) {
      return prevCount + 1;
    });
  };

  var _useReducer = (0, _react.useReducer)(taskReducer, {}, initTaskState),
      _useReducer2 = (0, _slicedToArray2["default"])(_useReducer, 2),
      state = _useReducer2[0],
      dispatch = _useReducer2[1];

  (0, _react.useEffect)(function () {
    if (staticConfig.isOnDemand && count === 0) {
      return noop;
    }

    var task = executeTaskOnEffect(dispatch, self.createTask, inputs, {
      onError: self.onError,
      onSuccess: self.onSuccess
    });
    self.task = task;
    return function () {
      task.cancel();
    };
  }, // eslint-disable-next-line react-hooks/exhaustive-deps
  [count].concat((0, _toConsumableArray2["default"])(staticConfig.isOnDemand ? [] : inputs)));
  return _objectSpread({
    execute: execute
  }, state, {}, self.task);
}

function useAsync(createTask) {
  var inputs = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
  var config = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  return useAsyncInternal(createTask, inputs, _objectSpread({}, config, {
    isOnDemand: false
  }));
}

function useAsyncOnDemand(createTask) {
  var inputs = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
  var config = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  return useAsyncInternal(createTask, inputs, _objectSpread({}, config, {
    isOnDemand: true
  }));
}

var _default = useAsync;
exports["default"] = _default;